# Standard Libraries
import subprocess

# pythonthesorimed
from pythonthesorimed.parseur import parseur

if __name__ == '__main__':
    parseur.main()
    subprocess.run(["yapf", "-i", "pythonthesorimed/api.py"])



# Standard Libraries
from collections.abc import Iterable
from exceptions import ThesorimedError

# Third Party Libraries
import psycopg2
from api import thesoapi
from psycopg2.extras import NamedTupleCursor


class ThesoItem:
    """
    Base classe interface pour thesorimed
    """

    @staticmethod
    def _connect():
        """
        Base fonction to connect to database.
        Return a pscipg connection
        should return psycopg2.connect
        """
        # raise NotImplementedError
        return psycopg2.connect(
            host="127.0.0.1", dbname="thesorimed", user="j", password="j")

    def _appel_char(self, obj_api, req):
        """
        Pour les précédure 'char' retournant une valeur simple
        """

        req = list(req)  # list to modify tupple
        if 'str' in obj_api.input_type[0]:  # turn arg to str if varchar
            req[0] = str(req[0])

        with self._connect() as con:
            with con.cursor(cursor_factory=NamedTupleCursor) as curs:
                curs.execute("SET search_path TO thesorimed, public")
                curs.callproc("thesorimed." + obj_api.name, req)
                res = curs.fetchone()
                cc = getattr(res, obj_api.name)
                return cc

    def _appel_refcursor(self, obj_api, req):
        """
        Pour les procédures "cursor" reournant un liste
        """
        # convert [int,int,int] to 'int,int, int'
        req = list(req)
        for i in range(len(req)):
            if 'str' in obj_api.input_type[i]:
                req[i] = ','.join(map(str, req[i]))
        # create connection
        with self._connect() as con:
            with con.cursor(cursor_factory=NamedTupleCursor) as curs:
                curs.execute("SET search_path TO thesorimed, public")
                curs.callproc("thesorimed." + obj_api.name, req)

                portal_name = curs.fetchone()[0]  # get the cursor
                portal_name = '"' + portal_name + '"'
                f = "FETCH ALL IN {0};".format(
                    portal_name)  # retrieve from cursor
                curs.execute(f)
                cc = curs.fetchall()
        return cc

    def _appel_proc(self, obj_api):
        if obj_api.genre == "char":
            return self._appel_char
        elif obj_api.genre == "cursor":
            return self._appel_refcursor
        else:
            raise ThesorimedError('Genre de la procédure inconnu')

    @staticmethod
    def _valide_req(obj, req):
        if len(req) != len(obj.input_type):
            raise ThesorimedError("Le nombre d'argument est invalide")

        if obj.input_type == ['str', 'int']:
            if not isinstance(req[0], Iterable):
                raise ThesorimedError("Le premier argument doit être iterable")
            for item in req[0]:
                try:
                    int(item)
                except ValueError:
                    raise ThesorimedError(
                        "L'argument attendu est une liste d'entier")

        for x, y in zip(obj.input_type, req):
            if x == 'int':
                if type(y) != int:
                    raise ThesorimedError("L'argument doit être un entier")
            import re
            longueur_champs = re.findall(r"(?:int|str)([0-9]+)", x)
            if longueur_champs:
                if y > pow(10, int(longueur_champs[0])):
                    raise ThesorimedError(
                        f"Longueur de requête limité à {longueur_champs}")

    @classmethod
    def proc(cls, name, *req):
        try:
            obj_api = thesoapi[name]
        except KeyError:
            raise ThesorimedError("La procédure appelé n'existe pas")

        cls._valide_req(obj_api, req)

        return cls._appel_proc(cls, obj_api)(cls, obj_api, req)


"""
usage dans Django:

from theso import ThesoItem

class Medicament:
    name
    cip

    detail(self, name, req):
        a = ThesoItem(self.cip)
        return a.proc(name, req)

    def monographie:
        retruen a.ThesoItem(self.cip).monographe


"""



# Standard Libraries
import copy
import io

# Third Party Libraries
import pytest
from parseur.parseur import ThesorimedApiParseur

from .gabarit import extracted, lines


@pytest.fixture(scope='function', autouse=True)
def parseur():
    return ThesorimedApiParseur(fichier=io.StringIO(lines), nb_api=4)


@pytest.fixture(scope='function', autouse=True)
def parseur_gabarit_extracted(parseur):
    p = ThesorimedApiParseur(fichier="lijl", nb_api=4)
    p.extracted = extracted
    return copy.deepcopy(p)


# @pytest.fixture(scope='session', autouse=True, tmpdir_factory)
#     parseur = (fichier=THESORIMED_API, save_api_dir=tmpdir_factory.getbasetemp().__str__())



intro = """
\"\"\"
Attention  : fichier autogénéré.
Ne pas l'éditer à la main.
Pour l'éditer, utiliser parseur
Il faut configurer host, dbn user et password à la main.

\"\"\"
"""

import_and_nametupple_def = """
# Standard Libraries
from collections import namedtuple

ProcApi = namedtuple('ProcApi', 'name input_type genre')


"""

base_func = """
def {name}(*req):
    return {appel}(\"{name}\", req, {params})
"""



"""
Attention  : fichier autogénéré.
Ne pas l'éditer à la main.
Pour l'éditer, utiliser parseur
Il faut configurer host, dbn user et password à la main.

"""

# Standard Libraries
from collections import namedtuple

ProcApi = namedtuple('ProcApi', 'name input_type genre')

thesoapi = {
    'get_cip':
    ProcApi(name='get_cip', input_type=['int6'], genre='char'),
    'get_frm':
    ProcApi(name='get_frm', input_type=['int6'], genre='char'),
    'get_voie':
    ProcApi(name='get_voie', input_type=['int6'], genre='char'),
    'is_atu':
    ProcApi(name='is_atu', input_type=['int6'], genre='char'),
    'is_retro':
    ProcApi(name='is_retro', input_type=['int6'], genre='char'),
    'is_t2a':
    ProcApi(name='is_t2a', input_type=['int6'], genre='char'),
    'get_cons':
    ProcApi(name='get_cons', input_type=['str13'], genre='char'),
    'is_hormono':
    ProcApi(name='is_hormono', input_type=['int6'], genre='char'),
    'get_the_gen_equiv':
    ProcApi(
        name='get_the_gen_equiv', input_type=['int', 'int'], genre='cursor'),
    'get_the_spe_details':
    ProcApi(
        name='get_the_spe_details', input_type=['str', 'int'], genre='cursor'),
    'get_the_cim10':
    ProcApi(name='get_the_cim10', input_type=['int'], genre='cursor'),
    'get_the_cipemg_id':
    ProcApi(
        name='get_the_cipemg_id', input_type=['int', 'str'], genre='cursor'),
    'get_the_incompat_fiche':
    ProcApi(name='get_the_incompat_fiche', input_type=['int'], genre='cursor'),
    'get_the_inter':
    ProcApi(name='get_the_inter', input_type=['str', 'int'], genre='cursor'),
    'get_the_conservation_spe':
    ProcApi(
        name='get_the_conservation_spe', input_type=['str'], genre='cursor'),
    'get_the_inter_sacinter_sac':
    ProcApi(
        name='get_the_inter_sacinter_sac',
        input_type=['str', 'str'],
        genre='cursor'),
    'get_the_cim10_by_code':
    ProcApi(name='get_the_cim10_by_code', input_type=['str'], genre='cursor'),
    'get_the_spe_by_ind_ucd':
    ProcApi(name='get_the_spe_by_ind_ucd', input_type=['str'], genre='cursor'),
    'get_the_spe_by_ind_ucdind':
    ProcApi(
        name='get_the_spe_by_ind_ucdind', input_type=['str'], genre='cursor'),
    'get_the_cipemg_text_sp':
    ProcApi(
        name='get_the_cipemg_text_sp',
        input_type=['int', 'str'],
        genre='cursor'),
    'get_the_hyper_code_by_name':
    ProcApi(
        name='get_the_hyper_code_by_name', input_type=['str'], genre='cursor'),
    'get_the_spe_same_cls':
    ProcApi(
        name='get_the_spe_same_cls', input_type=['int', 'int'],
        genre='cursor'),
    'get_the_code_cim10':
    ProcApi(name='get_the_code_cim10', input_type=['str'], genre='cursor'),
    'get_the_smr_asmr_spe':
    ProcApi(
        name='get_the_smr_asmr_spe', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_smr_asmr_gsp':
    ProcApi(
        name='get_the_smr_asmr_gsp', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_produit_stup':
    ProcApi(
        name='get_the_produit_stup', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_produit_stup_gsp':
    ProcApi(
        name='get_the_produit_stup_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_produit_dop':
    ProcApi(
        name='get_the_produit_dop', input_type=['str', 'int'], genre='cursor'),
    'get_the_produit_dop_gsp':
    ProcApi(
        name='get_the_produit_dop_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_reserve_hosp':
    ProcApi(
        name='get_the_reserve_hosp', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_reserve_hosp_gsp':
    ProcApi(
        name='get_the_reserve_hosp_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_prescr_rest':
    ProcApi(
        name='get_the_prescr_rest', input_type=['str', 'int'], genre='cursor'),
    'get_the_prescr_rest_gsp':
    ProcApi(
        name='get_the_prescr_rest_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_delivr_rest':
    ProcApi(
        name='get_the_delivr_rest', input_type=['str', 'int'], genre='cursor'),
    'get_the_delivr_rest_gsp':
    ProcApi(
        name='get_the_delivr_rest_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_gen_ref':
    ProcApi(name='get_the_gen_ref', input_type=['str', 'int'], genre='cursor'),
    'get_the_spe_dsp':
    ProcApi(name='get_the_spe_dsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_gsp_dsp':
    ProcApi(name='get_the_gsp_dsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_prix':
    ProcApi(name='get_the_prix', input_type=['str', 'int'], genre='cursor'),
    'get_the_atu':
    ProcApi(name='get_the_atu', input_type=['str', 'int'], genre='cursor'),
    'get_the_atu_gsp':
    ProcApi(name='get_the_atu_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_vigi_conduct':
    ProcApi(
        name='get_the_vigi_conduct', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_vigi_conduct_gsp':
    ProcApi(
        name='get_the_vigi_conduct_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_atc_ddd_v2':
    ProcApi(
        name='get_the_atc_ddd_v2', input_type=['str', 'int'], genre='cursor'),
    'get_the_atc_ddd_gsp_v2':
    ProcApi(
        name='get_the_atc_ddd_gsp_v2',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_solute':
    ProcApi(name='get_the_solute', input_type=['str', 'int'], genre='cursor'),
    'get_the_solute_gsp':
    ProcApi(
        name='get_the_solute_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_virtuel':
    ProcApi(name='get_the_virtuel', input_type=['str', 'int'], genre='cursor'),
    'get_the_forme_spe_v2':
    ProcApi(
        name='get_the_forme_spe_v2', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_forme_gsp':
    ProcApi(
        name='get_the_forme_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_compo_spe':
    ProcApi(
        name='get_the_compo_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_compo_gsp':
    ProcApi(
        name='get_the_compo_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_compo_synth_spe':
    ProcApi(
        name='get_the_compo_synth_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_compo_synth_gsp':
    ProcApi(
        name='get_the_compo_synth_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_effet_notoire_spe':
    ProcApi(
        name='get_the_effet_notoire_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_effet_notoire_gsp':
    ProcApi(
        name='get_the_effet_notoire_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_fic_info_thera':
    ProcApi(name='get_the_fic_info_thera', input_type=['str'], genre='cursor'),
    'get_the_fic_info_thera_gsp':
    ProcApi(
        name='get_the_fic_info_thera_gsp', input_type=['str'], genre='cursor'),
    'get_the_ter_cipemg_age':
    ProcApi(
        name='get_the_ter_cipemg_age',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_age_gsp':
    ProcApi(
        name='get_the_ter_cipemg_age_gsp',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_pds':
    ProcApi(
        name='get_the_ter_cipemg_pds',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_pds_gsp':
    ProcApi(
        name='get_the_ter_cipemg_pds_gsp',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_clr':
    ProcApi(
        name='get_the_ter_cipemg_clr',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_clr_gsp':
    ProcApi(
        name='get_the_ter_cipemg_clr_gsp',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_surf':
    ProcApi(
        name='get_the_ter_cipemg_surf',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_sur_gsp':
    ProcApi(
        name='get_the_ter_cipemg_sur_gsp',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_proc_spe':
    ProcApi(
        name='get_the_proc_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_proc_gsp':
    ProcApi(
        name='get_the_proc_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_proc_spev2':
    ProcApi(
        name='get_the_proc_spev2',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_proc_gspv2':
    ProcApi(
        name='get_the_proc_gspv2',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_sexe_spe':
    ProcApi(
        name='get_the_sexe_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_sexe_gsp':
    ProcApi(
        name='get_the_sexe_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_ter_cipemg_gr':
    ProcApi(
        name='get_the_ter_cipemg_gr',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_gr_gsp':
    ProcApi(
        name='get_the_ter_cipemg_gr_gsp',
        input_type=['str', 'int', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_al':
    ProcApi(
        name='get_the_ter_cipemg_al',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_ter_cipemg_al_gsp':
    ProcApi(
        name='get_the_ter_cipemg_al_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_spe_to_hyper':
    ProcApi(name='get_the_spe_to_hyper', input_type=['str'], genre='cursor'),
    'get_the_gsp_to_hyper':
    ProcApi(name='get_the_gsp_to_hyper', input_type=['str'], genre='cursor'),
    'get_the_gsp_to_hyper_sac':
    ProcApi(
        name='get_the_gsp_to_hyper_sac', input_type=['str'], genre='cursor'),
    'get_the_hyper_to_spe':
    ProcApi(
        name='get_the_hyper_to_spe', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_indic_spe':
    ProcApi(
        name='get_the_indic_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_indic_gsp':
    ProcApi(
        name='get_the_indic_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_cipe_spe_cim10':
    ProcApi(
        name='get_the_cipe_spe_cim10',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_cipe_gsp_cim10':
    ProcApi(
        name='get_the_cipe_gsp_cim10',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_indic_amm_ptt_tru':
    ProcApi(
        name='get_the_indic_amm_ptt_tru',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_ind_amm_ptt_tru_gsp':
    ProcApi(
        name='get_the_ind_amm_ptt_tru_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_inter_spe_sac':
    ProcApi(name='get_the_inter_spe_sac', input_type=['str'], genre='cursor'),
    'get_the_inter_gsp_sac':
    ProcApi(name='get_the_inter_gsp_sac', input_type=['str'], genre='cursor'),
    'get_the_incompat_spe_sac':
    ProcApi(
        name='get_the_incompat_spe_sac', input_type=['str'], genre='cursor'),
    'get_the_incompat_gsp_sac':
    ProcApi(
        name='get_the_incompat_gsp_sac', input_type=['str'], genre='cursor'),
    'get_the_pso_mm_by_spe':
    ProcApi(
        name='get_the_pso_mm_by_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_antibio_loc_spe':
    ProcApi(
        name='get_the_antibio_loc_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_antibio_syst_spe':
    ProcApi(
        name='get_the_antibio_syst_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_antibio_loc_gsp':
    ProcApi(
        name='get_the_antibio_loc_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_antibio_syst_gsp':
    ProcApi(
        name='get_the_antibio_syst_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_hypno_spe':
    ProcApi(
        name='get_the_hypno_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_hypno_gsp':
    ProcApi(
        name='get_the_hypno_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_anxio_spe':
    ProcApi(
        name='get_the_anxio_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_anxio_gsp':
    ProcApi(
        name='get_the_anxio_gsp', input_type=['str', 'int'], genre='cursor'),
    'get_the_anticanc_spe':
    ProcApi(
        name='get_the_anticanc_spe', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_anticanc_gsp':
    ProcApi(
        name='get_the_anticanc_gsp', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_index_vacc':
    ProcApi(
        name='get_the_index_vacc', input_type=['str', 'int'], genre='cursor'),
    'get_the_unit_prise':
    ProcApi(
        name='get_the_unit_prise', input_type=['str', 'int'], genre='cursor'),
    'get_the_medic_except_spe':
    ProcApi(
        name='get_the_medic_except_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_medic_except_gsp':
    ProcApi(
        name='get_the_medic_except_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_cfo_secable_spe':
    ProcApi(
        name='get_the_cfo_secable_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_admin_fo_solide_spe':
    ProcApi(
        name='get_the_admin_fo_solide_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_li_presc_spe':
    ProcApi(
        name='get_the_li_presc_spe', input_type=['str', 'int'],
        genre='cursor'),
    'get_the_cond_admin_spe':
    ProcApi(
        name='get_the_cond_admin_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_labo_exploit_spe':
    ProcApi(
        name='get_the_labo_exploit_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_pr_deriv_sang_spe':
    ProcApi(
        name='get_the_pr_deriv_sang_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_pr_deriv_sang_gsp':
    ProcApi(
        name='get_the_pr_deriv_sang_gsp',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_medic_orph_spe':
    ProcApi(
        name='get_the_medic_orph_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_pgr_surv_renf_spe':
    ProcApi(
        name='get_the_pgr_surv_renf_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_comm_cipemg_spe':
    ProcApi(
        name='get_the_comm_cipemg_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_redond_spe':
    ProcApi(
        name='get_the_redond_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_spe_indic_by_cim10':
    ProcApi(
        name='get_the_spe_indic_by_cim10',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_gsp_indic_by_cim10':
    ProcApi(
        name='get_the_gsp_indic_by_cim10',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_info_spe':
    ProcApi(
        name='get_the_info_spe', input_type=['str', 'int'], genre='cursor'),
    'get_the_etat_commer_spe':
    ProcApi(
        name='get_the_etat_commer_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_effet_inde_spe':
    ProcApi(
        name='get_the_effet_inde_spe',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_poso_v2':
    ProcApi(name='get_the_poso_v2', input_type=['str', 'int'], genre='cursor'),
    'get_the_compo_allsub_sp':
    ProcApi(
        name='get_the_compo_allsub_sp', input_type=['int'], genre='cursor'),
    'get_the_therap_anticanc':
    ProcApi(
        name='get_the_therap_anticanc',
        input_type=['str', 'int'],
        genre='cursor'),
    'get_the_delivre_max':
    ProcApi(name='get_the_delivre_max', input_type=['int'], genre='cursor'),
    'get_the_spe_is_or_bio':
    ProcApi(name='get_the_spe_is_or_bio', input_type=['str'], genre='cursor'),
    'get_the_spe_interop':
    ProcApi(name='get_the_spe_interop', input_type=['str'], genre='cursor'),
    'get_the_spe_hormono':
    ProcApi(
        name='get_the_spe_hormono', input_type=['str', 'int'], genre='cursor'),
    'get_the_gtiam_txt':
    ProcApi(name='get_the_gtiam_txt', input_type=['str'], genre='cursor'),
    'get_the_reserve_hosp_cip':
    ProcApi(
        name='get_the_reserve_hosp_cip',
        input_type=['str', 'int'],
        genre='cursor')
}



# Third Party Libraries
from parseur.parseur import ProcApi

lines = """
CREATE OR REPLACE FUNCTION GET_THE_SPE_DETAILS (VARCHAR,NUMERIC) RETURNS REFCURSOR AS
CREATE OR REPLACE FUNCTION thesorimed.get_cip (NUMERIC(6)) RETURNS character varying AS
fzefzef zef ze f zefz ef zef zef zef zefz ef
zefz efz efz efz efz ef
CREATE OR REPLACE FUNCTION thesorimed.get_frm (NUMERIC(6)) RETURNS character varying AS
CREATE OR REPLACE FUNCT
ION thesorimed.get_frm (NUMERIC(6)) RETURNS character varying AS
CREATE OR REPLACE FUNCTION GET_THE_ANXIO_SPE (VARCHAR,NUMERIC) RETURNS REFCURSOR AS"""

extracted = [
    ['get_the_spe_details', 'varchar,numeric', 'refcursor'],
    ['thesorimed.get_cip', 'numeric(6)', 'character varying'],
    ['thesorimed.get_frm', 'numeric(6)', 'character varying'],
    ['get_the_anxio_spe', 'varchar,numeric', 'refcursor'],
]

cleaned = [
    ['get_the_spe_details', ['str', 'int'], 'cursor'],
    ['get_cip', ['int6'], 'char'],
    ['get_frm', ['int6'], 'char'],
    ['get_the_anxio_spe', ['str', 'int'], 'cursor'],
]

thesoapi_test = {
    'get_the_spe_details':
    ProcApi(
        name='get_the_spe_details', input_type=['str', 'int'], genre='cursor'),
    'get_cip':
    ProcApi(name='get_cip', input_type=['int6'], genre='char'),
    'get_frm':
    ProcApi(name='get_frm', input_type=['int6'], genre='char'),
    'get_the_anxio_spe':
    ProcApi(
        name='get_the_anxio_spe', input_type=['str', 'int'], genre='cursor')
}



# Standard Libraries
import io
import re
from pathlib import Path

# Third Party Libraries
import pytest
from parseur.parseur import ThesorimedApiParseur
from parseur.prototype import import_and_nametupple_def, intro

from .gabarit import cleaned, extracted, lines, thesoapi_test


class TestThesorimedAPiParseur:
    def test_init(self, parseur):
        assert parseur.fichier

    def test_read_lines(self):
        p = ThesorimedApiParseur(fichier=io.StringIO(lines))

        p.read_lines()
        expected = [
            "CREATE OR REPLACE FUNCTION GET_THE_SPE_DETAILS (VARCHAR,NUMERIC) RETURNS REFCURSOR AS",
            "CREATE OR REPLACE FUNCTION thesorimed.get_cip (NUMERIC(6)) RETURNS character varying AS",
            "CREATE OR REPLACE FUNCTION thesorimed.get_frm (NUMERIC(6)) RETURNS character varying AS",
            "CREATE OR REPLACE FUNCTION GET_THE_ANXIO_SPE (VARCHAR,NUMERIC) RETURNS REFCURSOR AS"
        ]

        # x = [tuple(lst) for lst in p.lignes_lues]
        # y = [tuple(lst) for lst in expected]
        for x, y in zip(p.lignes_lues, expected):
            print(x, y.lower())
        for x, y in zip(p.lignes_lues, expected):
            assert x.rstrip() == y.lower()  #rstrip because of the trailing \n"

    def test_extraction(self, parseur):
        p = ThesorimedApiParseur(fichier=io.StringIO(lines))
        p.read_lines()
        p.extraction()

        for x, y in zip(p.extracted, extracted):
            assert x == y

    def test_read_lines_nb(self, parseur):
        parseur.read_lines()
        assert len(parseur.lignes_lues) == 4

    def test_LIGNE_MOTIF(self, parseur):
        assert parseur.ligne_motif.islower()

    def test_get_func_name(self, parseur):
        "verifice que chaque fonction trouvé est dans le fichier"
        parseur.read_lines()
        parseur.extraction()
        for x, y in zip(parseur.lignes_lues, parseur.extracted):
            assert re.search(re.escape(
                y[0]), x) is not None, "toutes fcontions  pas trouvéestrouvées"

    def test_get_params(self, parseur):
        parseur.read_lines()
        parseur.extraction()
        for x, y in zip(parseur.lignes_lues, parseur.extracted):
            print(x, y)
            assert re.search(re.escape(y[1]), x) is not None and re.search(
                re.escape(y[0]),
                x) is not None, "paramas presont pour bonne ligne"

    def test_get_retour(self, parseur):
        parseur.read_lines()
        parseur.extraction()
        for x, y in zip(parseur.lignes_lues, parseur.extracted):
            print(x, y)
            assert re.search(re.escape(y[2]), x) is not None and re.search(
                re.escape(y[0]),
                x) is not None, "retour presont pour bonne fonction"

    def test_clean_func_name(self, parseur_gabarit_extracted):
        parseur = parseur_gabarit_extracted
        parseur.clean_func_name()
        for i, j in zip(parseur.extracted, cleaned):
            assert i[0] == j[0]

    def test_clean_params(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.clean_params()
        for i, j in zip(p.extracted, cleaned):
            assert i[1] == j[1]

    def test_clean_retour(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.clean_retour()
        for i, j in zip(p.extracted, cleaned):
            assert i[2] == j[2]

    def test_clean_retour_raises(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.extracted[2][2] = "mok"  #on change le retourne
        with pytest.raises(Exception):
            p.clean_retour()

    def test_create_apicleaned(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.clean_func_name()
        p.clean_params()
        p.clean_retour()
        p.create_procapi()
        assert p.thesoapi == thesoapi_test

    def test_clean_all(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.clean_all()
        assert p.thesoapi == thesoapi_test

    def test_write_base(self, parseur_gabarit_extracted):
        p = parseur_gabarit_extracted
        p.clean_all()
        p.write_base()
        assert p.stream == intro + import_and_nametupple_def + "thesoapi = " + str(
            thesoapi_test)

    def test_write_to_file(self, parseur, tmpdir_factory):
        parseur.save_api_dir = tmpdir_factory.getbasetemp().__str__()
        parseur.thesoapi = thesoapi_test
        parseur.stream = "gf f hgj hg j hkk "
        parseur.write_to_file()
        a = Path(parseur.save_api_dir) / parseur.save_api_name
        assert a.read_text(), "file should be here usable"

    def test_create_thesorimed(self, tmpdir_factory):
        a = ThesorimedApiParseur(
            fichier=io.StringIO(lines),
            save_api_dir=tmpdir_factory.getbasetemp().__str__(),
            nb_api=4)
        a.create_thesorimed()
        b = Path(a.save_api_dir) / a.save_api_name
        assert b.read_text(
        ) == intro + import_and_nametupple_def + "thesoapi = " + str(
            thesoapi_test)



# Standard Libraries
import io
import re
from collections import namedtuple
from pathlib import Path

# pythonthesorimed
from pythonthesorimed.exceptions import ThesorimedParseurError

from .prototype import import_and_nametupple_def, intro

VERSION_API = 2.00

LIGNE_MOTIF = "CREATE OR REPLACE FUNCTION "

NB_API = 130  # dans la version 2, get_the_hyper_to_spe est défini 2 fois

SAVE_API_DIR = str(Path(__file__).parents[1])

SAVE_API_NAME = "api.py"

ProcApi = namedtuple("ProcApi", ['name', 'input_type', 'genre'])


class ThesorimedApiParseur:
    """
    Class générale permettant de parser l'api Thesorimed
    et la convertir en api python
    """

    def __init__(self,
                 fichier,
                 save_api_dir=SAVE_API_DIR,
                 ligne_motif=LIGNE_MOTIF,
                 save_api_name=SAVE_API_NAME,
                 nb_api=NB_API):
        """
        fichier : cheminr vers l'api.sql
        api : dictionnaire qui contiendra nom/params/retour des fonctions
        ligne_motif : LIGNE_MOTIF des parametres
        """
        self.fichier = fichier
        self.api = {}  # on initialise l'api de base
        self.ligne_motif = ligne_motif.lower()
        self.save_api_dir = save_api_dir
        self.nb_api = nb_api
        self.save_api_name = save_api_name

    def read_lines(self):
        """
        collecte chaque ligne de déclaration d'api selon motif
        """
        if isinstance(self.fichier, io.StringIO):  # monkeypatch dont like open
            f = self.fichier
        else:
            f = open(self.fichier)

        self.lignes_lues = []
        for ligne in f:
            ligne = ligne.lower()
            if ligne.startswith(self.ligne_motif):
                self.lignes_lues.append(ligne)

    def extraction(self):
        """
        extract [func_name, params, retour]
        """
        x = []
        for l in self.lignes_lues:
            a = []
            b = "CREATE OR REPLACE FUNCTION (.*) \(".lower()
            a.append(re.findall(b, l)[0])

            c = " \((.*)\) ".lower()
            a.append(re.findall(c, l)[0])

            d = "RETURNS (.*) AS".lower()
            a.append(re.findall(d, l)[0])

            x.append(a)
        self.extracted = x

    def clean_func_name(self):
        # remove "thesorimds"
        cle = 'thesorimed.'
        for f in self.extracted:
            if cle in f[0]:
                f[0] = f[0][len(cle):]

    def clean_params(self):
        # change numeric with int, varchar to str
        for f in self.extracted:
            sp = f[1].split(',')
            l = []
            for s in sp:
                x = re.findall("[0-9]+", s)
                x = x[0] if x else ''
                if 'numeric' in s:
                    l.append('int' + x)
                if 'varchar' in s:
                    l.append('str' + x)
            f[1] = l

    def clean_retour(self):
        #cursor or char
        for f in self.extracted:
            if 'refcursor' in f[2]:
                f[2] = "cursor"
            elif 'character' in f[2]:
                f[2] = "char"
            else:
                raise Exception(
                    "le Retour d'entrée n'est pas connu pour {} ".format(f[0]))

    def create_procapi(self):
        # turn extracted to namedtupple for convinience
        self.thesoapi = {}
        for i in self.extracted:
            # if i[0] in self.thesoapi:
            #     raise Exception(i[0])
            self.thesoapi[i[0]] = (ProcApi(i[0], i[1], i[2]))
        if len(self.thesoapi) != self.nb_api:
            raise ThesorimedParseurError(
                f'Le nombre Api attendu ({self.nb_api}) ne correspond pas : {len(self.thesoapi)}'
            )

    def clean_all(self):
        # run all cleaning function
        self.clean_func_name()

        self.clean_params()
        self.clean_retour()
        self.create_procapi()

    def write_base(self):
        # debut du fichier final
        chaine = intro + import_and_nametupple_def
        self.stream = chaine + "thesoapi = " + str(self.thesoapi)

    def write_to_file(self):
        path = Path(self.save_api_dir)
        path.mkdir(exist_ok=True)
        path = path / self.save_api_name
        path.write_text(self.stream)

    def write_api(self):
        self.write_base()
        self.write_to_file()

    def create_thesorimed(self):
        # run the whole process
        self.read_lines()
        self.extraction()
        self.clean_all()
        self.write_api()


def main():
    import sys
    a = ThesorimedApiParseur(fichier=sys.argv[1])
    a.create_thesorimed()
